import org.junit.Assert;
import org.junit.Test;

import java.util.ArrayList;

import static org.junit.Assert.*;

public class ArrazTest {

    private  int[] spiffyHandyIntArray = new int[] {4,5,102,6,-7,12,-32,92,8};
    private  Integer[] handyIntArray = new Integer[] {4,5,102,6,-7,12,-32,92,8};
    private Double[] spiffyHandyDoubleArray = new Double[] {1.0, 0.5, 3.6, 38.4, 17.3, 62.0, 9.0, 3.375, 0.0, 3.14159};
    Arraz arraz = new Arraz();

    @Test
    public void test1sumValuesOfArrays(){
        int actual = arraz.sumValuesOfArrays(spiffyHandyIntArray);
        Assert.assertEquals(190, actual);
    }

    @Test
    public void test2sumValuesOfArrays(){
        int[] arr = new int[0];
        int actual = arraz.sumValuesOfArrays(arr);
        Assert.assertEquals(0, actual);
    }

    @Test
    public void test1sumDoublesOfArrays(){
        Double actual = arraz.sumDoubleOfArrays(spiffyHandyDoubleArray);
        Assert.assertEquals(138.31650, actual, 1);
    }

    @Test
    public void test2sumDoublesOfArrays(){
        Double[] arr = new Double[0];
        Double actual = arraz.sumDoubleOfArrays(arr);
        Assert.assertEquals(0.0, actual);
    }

    @Test
    public void test1Average(){
        int actual = arraz.average(spiffyHandyIntArray);
        int expected = arraz.sumValuesOfArrays(spiffyHandyIntArray)/spiffyHandyIntArray.length;
        Assert.assertEquals(expected, actual);
    }

    @Test (expected = ArithmeticException.class)
    public void test2Average(){
        int[] arr = new int[0];
        int actual = arraz.average(arr);
    }

    @Test
    public void test1AverageDbl(){
        Double actual = arraz.averageDbl(spiffyHandyDoubleArray);
        Double expected = arraz.sumDoubleOfArrays(spiffyHandyDoubleArray)/spiffyHandyDoubleArray.length;
        Assert.assertEquals(expected, actual);
    }

    @Test (expected = NullPointerException.class)
    public void test2AverageDbl(){
        Double[] arr = new Double[1];
        Double actual = arraz.averageDbl(arr);
        Assert.assertEquals(null, actual);
    }

    @Test
    public void test1ContainsValue(){
        Assert.assertTrue(arraz.containsValue(spiffyHandyIntArray, 5));
    }

    @Test
    public void test2ContainsValue(){
        Assert.assertFalse(arraz.containsValue(spiffyHandyIntArray, 2));
    }

    @Test
    public void test1ReverseArray(){
        int[] expected = new int[] {8,92,-32,12,-7,6,102,5,4};
        int[] actual = arraz.reverse(spiffyHandyIntArray);
        for (int i = 0; i <actual.length ; i++) {
            Assert.assertEquals(expected[i],actual[i]);
        }
    }

    @Test
    public void test2ReverseArray(){
        int[] expected = new int[] {1,2,3};
        int[] actual = arraz.reverse(expected);
        for (int i = 0; i <actual.length ; i++) {
            Assert.assertEquals(expected[i],actual[i]);
        }
    }

    @Test
    public void test1getOddEvensOfArray(){
        OddEven actual = arraz.getOddEvensOfArray(spiffyHandyIntArray);
        OddEven expected = new OddEven(2,7);
        Assert.assertEquals(expected.getEvens(), actual.getEvens());
    }

    @Test
    public void test2OddEvensOfArray(){
        OddEven actual = arraz.getOddEvensOfArray(spiffyHandyIntArray);
        OddEven expected = new OddEven(2,7);
        Assert.assertEquals(expected.getOdds(),actual.getOdds());
    }

    @Test
    public void test1FindIndexOf(){
        Assert.assertEquals(arraz.findIndexOf(spiffyHandyIntArray, 5), 1);
    }

    @Test
    public void test2FindIndexOf(){
        Assert.assertEquals(arraz.findIndexOf(spiffyHandyIntArray, 1), -1);
    }

    @Test
    public void test1CopyArrayByIterator(){
        Integer[] actual = arraz.copyArrayByIterator(handyIntArray);
        Integer[] arr = new Integer[] {4,5,102,6,-7,12,-32,92,8};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test2CopyArrayByIterator(){
        Integer[] actual = arraz.copyArrayByIterator(handyIntArray);
        Integer[] expected = new Integer[] {4,5,102,6,-7,12,-32,92,8};
        Assert.assertNotSame(expected,actual);
    }

    @Test
    public void test1CopyArrayByLoop(){
        int[] actual = arraz.copyArrayByLoop(spiffyHandyIntArray);
        int[] arr = new int[] {4,5,102,6,-7,12,-32,92,8};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test2CopyArrayByLoop(){
        int[] actual = arraz.copyArrayByLoop(spiffyHandyIntArray);
        int[] expected = new int[] {4,5,102,6,-7,12,-32,92,8};
        Assert.assertNotSame(expected,actual);
    }

    @Test
    public void test1RemoveElement(){
        int[] arr = arraz.removeElement(spiffyHandyIntArray, -7);
        Assert.assertFalse(arraz.containsValue(arr, -7));
    }

    @Test
    public void test2RemoveElement(){
        int [] arr = arraz.removeElement(spiffyHandyIntArray, 4);
        Assert.assertFalse(arraz.containsValue(arr, 4));
    }

    @Test
    public void test1InsertIntoArrayAt(){
        int[] arr = arraz.insertIntoArrayAt(spiffyHandyIntArray,2,2);
        Assert.assertEquals(arr[2], 2);
    }

    @Test(expected = ArrayIndexOutOfBoundsException.class)
    public void test2InsertIntoArrayAt(){
        int[] arr = arraz.insertIntoArrayAt(spiffyHandyIntArray,2,25);
    }

    @Test
    public void test1FindMinMax(){
        MinMax actual = arraz.findMinMax(spiffyHandyIntArray);
        MinMax  expected = new MinMax(-32,102);
        Assert.assertEquals(expected.getMax(), actual.getMax());
    }

    @Test
    public void test2FindMinMax(){
        MinMax actual = arraz.findMinMax(spiffyHandyIntArray);
        MinMax  expected = new MinMax(-32,102);
        Assert.assertEquals(expected.getMin(), actual.getMin());
    }

    @Test
    public void test1RemoveDupes(){
        Integer[] arr = new Integer[]{3,3,3,5,2,8};
        arr = arraz.removeDupes(arr);
        Assert.assertEquals(arr.length, 4);
    }

    @Test
    public void test2RemoveDupes(){
        Integer[] arr = new Integer[]{4,3,5,2,8,2,5};
        arr = arraz.removeDupes(arr);
        Assert.assertEquals(arr.length, 5);
    }

    @Test
    public void test1Find2ndLargest(){
        int result = arraz.find2ndLargest(spiffyHandyIntArray);
        Assert.assertEquals(92, result);
    }

    @Test
    public void test2Find2ndLargest(){
        int[] arr = new int[]{-4,-3,-5,-2,-8,-2,-5};
        int result = arraz.find2ndLargest(arr);
        Assert.assertEquals(-2, result);
    }

    @Test
    public void test1MakeArrayList(){
        ArrayList<Integer> list = arraz.makeArrayList(spiffyHandyIntArray);
        Assert.assertEquals(list.getClass(), ArrayList.class);
    }

    @Test
    public void test2MakeArrayList(){
        ArrayList<Integer> list = arraz.makeArrayList(spiffyHandyIntArray);
        Assert.assertEquals(spiffyHandyIntArray.length, list.size());
    }

    @Test
    public void test1MakeArray(){
        ArrayList<Integer> list = new ArrayList<Integer>();
        list.add(4);
        list.add(3);
        list.add(9);
        list.add(8);
        int[] arr = arraz.makeArray(list);
        Assert.assertEquals(arr.getClass(), int[].class);
    }

    @Test
    public void test2MakeArray(){
        ArrayList<Integer> list = new ArrayList<Integer>();
        list.add(4);
        list.add(3);
        list.add(9);
        list.add(8);
        int[] arr = arraz.makeArray(list);
        Assert.assertEquals(list.size(), arr.length);
    }

    @Test
    public void test1Check2Arrays(){
        int[] testArr = new int[] {4,5,102,6,-7,12,-32,92,8};
        boolean actual = arraz.check2Arrays(testArr,spiffyHandyIntArray);
        Assert.assertTrue(actual);
    }

    @Test
    public void test2Check2Arrays(){
        int[] testArr = new int[] {4,5,102,6,-7,44,-32,92,8};
        boolean actual = arraz.check2Arrays(testArr,spiffyHandyIntArray);
        Assert.assertFalse(actual);
    }

    @Test
    public void test1AverageWithoutMinMax(){
        int actual = arraz.averageWithoutMinMax(spiffyHandyIntArray);
        Assert.assertEquals(17, actual);
    }

    @Test
    public void test2AverageWithoutMinMax(){
        int[] testArr = new int[] {4,3,2,6,7,4,1};
        int actual = arraz.averageWithoutMinMax(testArr);
        Assert.assertEquals(3, actual);
    }

    @Test
    public void test1Has65and77(){
        int[] testArr = new int[] {4,3,2,6,7,4,1};
        boolean actual = arraz.has65and77(testArr);
        Assert.assertFalse(actual);
    }

    @Test
    public void test2Has65and77(){
        int[] testArr = new int[] {4,3,65,2,6,7,77,4,1};
        boolean actual = arraz.has65and77(testArr);
        Assert.assertTrue(actual);
    }

    @Test
    public void test1TotalOf10sIs30(){
        int[] testArr = new int[] {4,3,65,2,6,7,77,4,1};
        boolean actual = arraz.toalOf10sIs30(testArr);
        Assert.assertFalse(actual);
    }

    @Test
    public void test2TotalOf10sIs30(){
            int[] testArr = new int[] {4,3,10,2,6,7,10,4,10,1};
            boolean actual = arraz.toalOf10sIs30(testArr);
            Assert.assertTrue(actual);
    }

    @Test
    public void test1makeCopyReverse(){
        int[] actual = arraz.makeCopyReverse(spiffyHandyIntArray);
        int[] arr = new int[] {8,92,-32,12,-7,6,102,5,4};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test2makeCopyReverse(){
        int[] testArr = new int[] {4,3,2,6,7,4,1};
        int[] actual = arraz.makeCopyReverse(testArr);
        int[] arr = new int[] {1,4,7,6,2,3,4};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test1removeLastAndCopy(){
        int[] testArr = new int[] {4,3,2,33,7,4,1};
        int[] actual = arraz.removeLastAndCopy(testArr);
        int[] arr = new int[] {4,3,2,33,7,4};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test2removeLastAndCopy(){
        int[] testArr = new int[] {4,3,10,2,6,7,10,4,10,1};
        int[] actual = arraz.removeLastAndCopy(testArr);
        Assert.assertEquals(testArr.length-1,actual.length);
    }

    @Test
    public void test1removeFirstAndCopy(){
        int[] testArr = new int[] {4,3,2,33,7,9,1};
        int[] actual = arraz.removeFirstAndCopy(testArr);
        int[] arr = new int[] {3,2,33,7,9,1};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test2removeFirstAndCopy(){
        int[] testArr = new int[] {4,3,10,2,6,7,10,4,10,1};
        int[] actual = arraz.removeFirstAndCopy(testArr);
       Assert.assertEquals(testArr.length-1, actual.length);
    }

    @Test
    public void test1insertFirstAndCopy(){
        int[] testArr = new int[] {4,3,2,33,7,9,1};
        int[] actual = arraz.insertFirstAndCopy(testArr, 1);
        int[] arr = new int[] {1,4,3,2,33,7,9,1};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test2insertFirstAndCopy(){
        int[] testArr = new int[] {4,3,10,2,6,7,10,4,10,1};
        int[] actual = arraz.insertFirstAndCopy(testArr,2);
        Assert.assertEquals(testArr.length+1, actual.length);
    }

    @Test
    public void test1insertEndAndCopy(){
        int[] testArr = new int[] {4,3,2,33,7,9,1};
        int[] actual = arraz.insertEndAndCopy(testArr, 1);
        int[] arr = new int[] {4,3,2,33,7,9,1,1};
        for (int i=0; i<arr.length; i++){
            Assert.assertEquals(arr[i], actual[i]);
        }
    }

    @Test
    public void test2insertEndAndCopy(){
        int[] testArr = new int[] {4,3,10,2,6,7,10,4,10,1};
        int[] actual = arraz.insertEndAndCopy(testArr,2);
        Assert.assertEquals(testArr.length+1, actual.length);
    }

    @Test
    public void test1SortArrayEvenOdd(){
        int[] spiffyHandyIntArray = new int[] {4,5,102,6,-7,12,-32,92,8};
        int[] actual = arraz.sortArrayEvenOdd(spiffyHandyIntArray);
        int[] expected = new int[]{4,102,6,12,-32,92,8,5,-7};
        for (int i = 0; i < actual.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }
    }

    @Test
    public void test2SortArrayEvenOdd(){
        int[] testArr = new int[] {4,3,10,2,6,7,10,4,10,1};
        int[] actual = arraz.sortArrayEvenOdd(testArr);
        int[] expected = new int[]{4,10,2,6,10,4,10,3,7,1};
        for (int i = 0; i < actual.length; i++) {
            Assert.assertEquals(expected[i], actual[i]);
        }
    }

}